﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq.Expressions;
using System.Windows;
using Infragistics.Olap;
using Infragistics.Olap.FlatData;

namespace PivotGridTestApp
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            pivotGrid.Loaded += FlatDataLoaded;
        }

        private void FlatDataLoaded(object sender, RoutedEventArgs e)
        {
            var flatDataSource = new FlatDataSource
            {
                ItemsSource = CreateCollection(),
                Cube = DataSourceBase.GenerateInitialCube("PivotGridData"),
                Rows = DataSourceBase.GenerateInitialItems("[Tenor].[Tenor]"),
                Columns = DataSourceBase.GenerateInitialItems("[Currency].[Currency]"),
                Measures = DataSourceBase.GenerateInitialItems("Value")
            };

            Expression<Func<PivotGridData, DateTime>> tenorExpression = result => EvaluateTenorSort(result.Tenor);

            var tenorHierarchy = new HierarchyDescriptor { SourcePropertyName = "Tenor" };
            var level = new HierarchyLevelDescriptor
            {
                LevelName = "Tenor",
                LevelExpressionPath = "Tenor"
            };
            tenorHierarchy.LevelDescriptors.Add(level);

            tenorHierarchy.LevelDescriptors[0].OrderByKeyExpression = tenorExpression;
            flatDataSource.AddHierarchyDescriptor(tenorHierarchy);

            pivotGrid.DataSource = flatDataSource;
            dataSelector.DataSource = flatDataSource;
        }

        private IEnumerable CreateCollection()
        {
            var products = new List<PivotGridData>();
            var product = new PivotGridData("10Y", "Swap", "GBP", 111);
            products.Add(product);
            var product1 = new PivotGridData("5Y", "Swap", "GBP", 222);
            products.Add(product1);
            var product2 = new PivotGridData("2Y", "Swap", "GBP", 333);
            products.Add(product2);
            var product3 = new PivotGridData("1Y", "Swap", "GBP", 444);
            products.Add(product3);
            var product4 = new PivotGridData("50Y", "Swap", "GBP", 555);
            products.Add(product4);
            var product5 = new PivotGridData("60Y", "Swap", "GBP", 666);
            products.Add(product5);
            var product6 = new PivotGridData("100Y", "Swap", "GBP", 777);
            var product7 = new PivotGridData("10D", "Swap", "GBP", 111);
            products.Add(product7);
            var product8 = new PivotGridData("5D", "Swap", "GBP", 111);
            products.Add(product8);
            var product9 = new PivotGridData("5M", "Swap", "GBP", 111);
            products.Add(product9);
            products.Add(product6);
            return products;
        }

        public static DateTime EvaluateTenorSort(string tenor)
        {
            var date = DateTime.MinValue;
            if (tenor == null) return date;
            int tenorNum;
            if (tenor.ToUpper().Contains("D"))
            {
                int.TryParse(tenor.Substring(0, tenor.ToUpper().IndexOf('D')), out tenorNum);
                return date.AddDays(tenorNum);
            }
            if (tenor.ToUpper().Contains("W"))
            {
                int.TryParse(tenor.Substring(0, tenor.ToUpper().IndexOf('W')), out tenorNum);
                return date.AddDays(tenorNum * 7);
            }
            if (tenor.ToUpper().Contains("M"))
            {
                int.TryParse(tenor.Substring(0, tenor.ToUpper().IndexOf('M')), out tenorNum);
                return date.AddMonths(tenorNum);
            }
            if (tenor.ToUpper().Contains("Y"))
            {
                int.TryParse(tenor.Substring(0, tenor.ToUpper().IndexOf('Y')), out tenorNum);
                return date.AddYears(tenorNum);
            }
            return date;
        }

       
    }
}
